package net.wanji.utc.task;

import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.common.collect.Maps;
import lombok.extern.slf4j.Slf4j;
import net.jodah.expiringmap.ExpirationPolicy;
import net.jodah.expiringmap.ExpiringMap;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.common.utils.tool.JacksonUtils;
import net.wanji.databus.dao.entity.CrossLightsStatusHistPO;
import net.wanji.databus.dao.mapper.CrossInfoMapper;
import net.wanji.databus.dao.mapper.CrossLightsStatusHistMapper;
import net.wanji.databus.dao.mapper.ManufacturerInfoMapper;
import net.wanji.databus.po.CrossInfoPO;
import net.wanji.databus.po.ManufacturerInfoPO;
import net.wanji.databus.vo.LightsStatusVO;
import net.wanji.databus.vo.LightsStatusVO2;
import net.wanji.feign.service.UtcDTFeignClients;
import net.wanji.feign.service.UtcHisenseFeignClients;
import net.wanji.utc.common.constant.Constants;
import net.wanji.utc.common.typeenum.BasicEnum;
import net.wanji.utc.dto.twinspat.Intersection;
import net.wanji.utc.dto.twinspat.Phase;
import net.wanji.utc.dto.twinspat.WHSpatData;
import net.wanji.utc.kafka.ProducerHandler;
import net.wanji.utc.service.runninginfo.HkLightsStatusService;
import net.wanji.utc.service.runninginfo.SignalStatusService;
import net.wanji.utc.util.DirectionConverter;
import net.wanji.utc.util.TurnDirectionMapper;
import net.wanji.utc.websocket.RealTimeDataWebSocket;
import org.apache.commons.lang3.StringUtils;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.elasticsearch.action.index.IndexRequest;
import org.elasticsearch.action.index.IndexResponse;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.client.indices.CreateIndexRequest;
import org.elasticsearch.client.indices.CreateIndexResponse;
import org.elasticsearch.client.indices.GetIndexRequest;
import org.elasticsearch.common.xcontent.XContentType;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.kafka.annotation.KafkaListener;
import org.springframework.kafka.support.Acknowledgment;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * 订阅信号运行状态、告警、灯态
 *
 * @author Kent HAN
 * @date 2022/11/21 9:01
 */
@Component
@Slf4j
@SuppressWarnings("all")
public class SignalStatusTask {
    @Autowired
    SignalStatusService signalStatusService;
    @Resource(name = "commonThreadPoolExecutor")
    ThreadPoolTaskExecutor commonThreadPoolExecutor;

    @Resource
    private HkLightsStatusService hkLightsStatusService;
    @Resource
    private ManufacturerInfoMapper manufacturerInfoMapper;
    @Resource
    private CrossInfoMapper crossInfoMapper;
//    @Resource
    private UtcDTFeignClients utcDTFeignClients;
    @Resource
    private CrossLightsStatusHistMapper crossLightsStatusHistMapper;
    @Autowired
    private ProducerHandler producerHandler;
    @Resource
    RestHighLevelClient client;

    private static final Map<String, Integer> runModeTimeMap = new ConcurrentHashMap<>();
    @Qualifier("net.wanji.feign.service.UtcHisenseFeignClients")
    @Autowired
    private UtcHisenseFeignClients utcHisenseFeignClients;

    private static final ConcurrentHashMap<String, List<LightsStatusVO2>> produceListMap = new ConcurrentHashMap<>();

    //路口信息过期map<路口id,路口信息>
    public static final ExpiringMap<String, CrossInfoPO> crossCacheMap = ExpiringMap.builder()
            .expiration(1, TimeUnit.MINUTES)
            .maxSize(100)
            .variableExpiration()
            .expirationPolicy(ExpirationPolicy.CREATED)
            //过期监听
            .asyncExpirationListener((rcuIdStr, value) -> {
            }).build();
    //信号灯倒计时缓存map<crossid_相位id, 倒计时>
    public static final ExpiringMap<String, Integer> phaseCountdownCacheMap = ExpiringMap.builder()
            .expiration(60 * 24, TimeUnit.MINUTES)
            .maxSize(100)
            .variableExpiration()
            .expirationPolicy(ExpirationPolicy.CREATED)
            //过期监听
            .asyncExpirationListener((rcuIdStr, value) -> {
            }).build();

    // 运行状态、告警，1分钟一次
    @Scheduled(fixedRate = 60 * 1000)
    public void runningStatusAlarm() throws Exception {
        signalStatusService.runningStatusAlarm();
    }

    /**
     * 消费孪生灯态kafka数据
     */
    @KafkaListener(topics = {"WHSpatData"}, groupId = "utc-whspat-consumer-01")
    public void consumeTwinSpat(ConsumerRecord<Object, Object> record, Acknowledgment acknowledgment) {
        try {
            String lightStatusJson = String.valueOf(record.value());
            WHSpatData light = JSONObject.parseObject(lightStatusJson, WHSpatData.class);
            if (Objects.nonNull(light)) {
                //--------------------------------------------------------------------------
                ConcurrentHashMap<String, RealTimeDataWebSocket> evenWarnWebSocketMap = RealTimeDataWebSocket.getEvenWarnWebSocketMap();
                if (!evenWarnWebSocketMap.isEmpty()) {
                    //1. 按路口分组后的websocket连接列表：map<crossId, List<Map.Entry<String, RealTimeDataWebSocket>>>
                    Map<String, List<Map.Entry<String, RealTimeDataWebSocket>>> crossEntryMap = getCrossEntryMap(evenWarnWebSocketMap);

                    //2. 循环每一个路口，并且循环每一个websocket连接，发送灯态数据
                    for (Map.Entry<String, List<Map.Entry<String, RealTimeDataWebSocket>>> entry : crossEntryMap.entrySet()) {
                        ObjectMapper mapper = JacksonUtils.getInstance();
                        String crossId = entry.getKey();
                        //2.1 获取路口信息
                        CrossInfoPO crossInfoPO = crossCacheMap.get(crossId);
                        if (Objects.isNull(crossInfoPO)) {
                            crossInfoPO = crossInfoMapper.selectByPrimaryKey(crossId);
                            if (Objects.isNull(crossInfoPO)) {
                                continue;
                            }
                            crossCacheMap.put(crossId, crossInfoPO);
                        }
                        //2.2 获取灯态信息，传递参数：mapper,crossId,路口信息
                        List<LightsStatusVO> lightsStatusVOS = getLightsStatusVOS2(crossInfoPO, light);

                        //2.3 保存灯态数据并发送
                        saveAndSendLightsStatus(mapper, crossId, lightsStatusVOS, entry);
                    }
                    sendKafka();
                }
                //--------------------------------------------------------------------------

            }

        }catch (Exception e) {
            log.error("解析灯态数据异常", e);
        }
    }

    // 灯态，每秒一次
//    @Scheduled(fixedRate = 1000)
    public void lightStatus() {
        ConcurrentHashMap<String, RealTimeDataWebSocket> evenWarnWebSocketMap = RealTimeDataWebSocket.getEvenWarnWebSocketMap();
        try {
            if (!evenWarnWebSocketMap.isEmpty()) {
                //1. 按路口分组后的websocket连接列表：map<crossId, List<Map.Entry<String, RealTimeDataWebSocket>>>
                Map<String, List<Map.Entry<String, RealTimeDataWebSocket>>> crossEntryMap = getCrossEntryMap(evenWarnWebSocketMap);

                //2. 循环每一个路口，并且循环每一个websocket连接，发送灯态数据
                for (Map.Entry<String, List<Map.Entry<String, RealTimeDataWebSocket>>> entry : crossEntryMap.entrySet()) {
                    //commonThreadPoolExecutor.execute(() -> {
                    ObjectMapper mapper = JacksonUtils.getInstance();
                    String crossId = entry.getKey();
                    //2.1 获取路口信息
                    CrossInfoPO crossInfoPO = crossInfoMapper.selectByPrimaryKey(crossId);
                    if (Objects.isNull(crossInfoPO)) {
                        continue;
                    }
                    //2.2 获取灯态信息，传递参数：mapper,crossId,路口信息
                    List<LightsStatusVO> lightsStatusVOS = getLightsStatusVOS(mapper, crossId, crossInfoPO);

                    //2.3 保存灯态数据并发送
                    saveAndSendLightsStatus(mapper, crossId, lightsStatusVOS, entry);
                    //});
                }
                sendKafka();
                //sendElastic();
            }
        } catch (Exception e) {
            log.error("实时推送灯态数据失败", e);
        }
    }

    /**
     * 保存灯态数据并发送
     *
     * @param mapper
     * @param crossId
     * @param lightsStatusVOS
     */
    private void saveAndSendLightsStatus(ObjectMapper mapper, String crossId, List<LightsStatusVO> lightsStatusVOS,
                                         Map.Entry<String, List<Map.Entry<String, RealTimeDataWebSocket>>> entry) {
        //1. 获取路口websocket连接列表
        List<Map.Entry<String, RealTimeDataWebSocket>> value = entry.getValue();
        produceListMap.remove(crossId); // 清除上一秒缓存

        if (!CollectionUtils.isEmpty(lightsStatusVOS)) {
            //2. 循环遍历灯态数据，并保存灯态数据到数据库，并发送灯态数据到websocket
            for (LightsStatusVO lightsStatusVO : lightsStatusVOS) {
                if (Objects.nonNull(lightsStatusVO) && Objects.equals(lightsStatusVO.getCrossId(), crossId)) {
                    //
                    calculateControlCountDown(crossId, lightsStatusVO);
                    try {
                        // 相同路口不同websocket统一发送灯态
                        List<LightsStatusVO2> listResult2 = convertDirLampGroupMap2Entity(lightsStatusVOS);
                        String json = mapper.writeValueAsString(listResult2);
                        for (Map.Entry<String, RealTimeDataWebSocket> socketEntry : value) {
                            String crossIdStr = socketEntry.getKey();
                            RealTimeDataWebSocket webSocket = socketEntry.getValue();
                            //  发送数据到websocket
                            webSocket.sendInfo(json, crossIdStr);
                        }
                        insertIntoHist(crossId, json);
                        produceListMap.put(crossId, listResult2);
                    } catch (Exception e) {
                        log.error("RealTimeDataWebSocket发送异常，异常信息：", e);
                    }
                }
            }
        }
    }

    /**
     * 获取不同厂商的灯态列表
     *
     * @param mapper
     * @param crossId
     * @param manufacturerInfoPO
     * @return
     */
    private List<LightsStatusVO> getLightsStatusVOS(ObjectMapper mapper, String crossId, CrossInfoPO crossInfoPO) {
        try {
            ManufacturerInfoPO manufacturerInfoPO = manufacturerInfoMapper.selectById(crossInfoPO.getManufacturerId());
            if (StringUtils.equals(BasicEnum.ManufacturerEnum.HK.getCode(), manufacturerInfoPO.getCode())) {
                return hkLightsStatusService.getHkLightsStatus(crossId);
            }
            if (StringUtils.equals(BasicEnum.ManufacturerEnum.DT.getCode(), manufacturerInfoPO.getCode())) {
                JsonViewObject jsonViewObject = utcDTFeignClients.lightStatus(crossId);
                return mapper.convertValue(jsonViewObject.getContent(), new TypeReference<List<LightsStatusVO>>() {
                });
            }
            if (StringUtils.endsWithIgnoreCase(BasicEnum.ManufacturerEnum.HISENSE.getCode(), manufacturerInfoPO.getCode())) {
                JsonViewObject jsonViewObject = utcHisenseFeignClients.lightStatus(crossId);
                return mapper.convertValue(jsonViewObject.getContent(), new TypeReference<List<LightsStatusVO>>() {
                });
            }
            if (StringUtils.endsWithIgnoreCase("STATIC", manufacturerInfoPO.getCode())) {
                LightsStatusVO lightsStatusVO = SignalStatus4StaticSchemeTask.currentCrossLightStatusCache.get(crossId);
                List<LightsStatusVO> lightsStatusVOS = new ArrayList<>();
                if (Objects.nonNull(lightsStatusVO)) {
                    lightsStatusVOS.add(lightsStatusVO);
                }
                return lightsStatusVOS;
            }
        } catch (Exception e) {
            log.error("获取当前厂商路口编号[{}]灯态异常", crossId, e);
        }
        return Collections.emptyList();
    }

    public static Map<String, String> crossIdMap = new HashMap();
    {
        crossIdMap.put("R-081507", "12Q5V099C30");
    }
    /**
     * 获取路口灯态列表
     * @param crossInfoPO 路口信息
     * @param light 孪生灯态数据
     * @return
     */
    private List<LightsStatusVO> getLightsStatusVOS2(CrossInfoPO crossInfoPO, WHSpatData light){
        List<LightsStatusVO> result = new ArrayList<>();

        //1. 获取路口列表
        List<Intersection> intersections = light.getIntersections();

        //2. 循环每一个路口（实际就一个路口，因为rsuId对应一个路口）
        for (Intersection nodeCross : intersections) {
            List<Phase> phases = nodeCross.getPhases();
            String crossId = (String) crossIdMap.get(light.getRsuId());

            //转向-方向-灯态
            Map<String, Boolean> m  = Maps.newHashMap();
            //循环每一个相位，获取该相位相关字段：如周期时间、周期倒计时、相位灯态倒计时、方向、车道转向、灯态
            for (Phase p : phases) {
                LightsStatusVO infoVo = new LightsStatusVO();
                infoVo.setRunMode("8");// 控制模式
                infoVo.setPhasePlanId("1");// 相位方案号
                infoVo.setSchemeId("1");
                infoVo.setCode(light.getRsuId());
                infoVo.setCrossId(crossId);
                infoVo.setCycleCountDown(0);

                String cycleLenKey = crossId + "_" + p.getPhaseId();
                Integer len = phaseCountdownCacheMap.get(cycleLenKey);
                if (len != null){
                    if (len <  p.getLimitEndTime()){
                        phaseCountdownCacheMap.put(cycleLenKey, p.getLimitEndTime());
                        infoVo.setCycleLen(p.getLimitEndTime());
                    }else{
                        infoVo.setCycleLen(len);
                    }
                }else{
                    phaseCountdownCacheMap.put(cycleLenKey, p.getLimitEndTime());
                }
                infoVo.setPhaseId(p.getPhaseId() + "");
                infoVo.setCyclePhaseCountDown(p.getLimitEndTime());
                //map<方向,倒计时>
                Map<String, Object> phaseMap = new HashMap<>();
                infoVo.setPhaseMap(phaseMap);

                //后两个参数是map<方向,倒计时>, 倒计时
                Map<String, Object> dirLampGroupMap = getDirLampGroupMap(p, m, phaseMap, p.getLimitEndTime());
                infoVo.setDirLampGroupMap(dirLampGroupMap);
                infoVo.setTimeStamp(light.getTimestamp() + "");
                infoVo.setSchemeStartTime(light.getTimestamp() + "");

                result.add(infoVo);
            }
        }
        return result;
    }

    private Map<String, Object> getDirLampGroupMap(Phase phase, Map<String, Boolean> m, Map<String, Object> phaseMap, int limitEndTime) {
        //Map<进口方向, Map<车道转向, 对应灯态>>
        Map<String, Object> dirLampGroupMap = new HashMap<>();

        //进口方向
        int importDirect = phase.getImportDirect();
        int dir = DirectionConverter.convertDirection(importDirect);//进口方向适配
        phaseMap.put(dir + "", limitEndTime);//方向，倒计时

        //Map<车道转向, 对应灯态>
        Map<Integer, String> turnMap = new HashMap<>();
        dirLampGroupMap.put(dir + "", turnMap);

        //获取灯态
        // 信号灯颜色   [0..8]，0：未知状态，1：信号灯未工作，2：红闪，3：红灯状态，4：绿灯待行状态，
        // 5：绿灯状态，6：受保护相位绿灯（箭头灯），7：黄灯状态，8：黄闪。
        int light = phase.getLight();
        String color = transferLampGroup(light);//车道灯态适配
        int turn = phase.getTurn();
        int turnDir = TurnDirectionMapper.mapToTargetValue(turn);//车道转向适配

        if (m.containsKey(dir + "-" +  turnDir + "-" + color)){
            return null;
        }else {
            m.put(dir + "-" + turnDir + "-" + color, true);
        }
        turnMap.put(turnDir, color);
        return dirLampGroupMap;
    }
    /**
     * 信号灯状态转颜色
     * 信号灯颜色   [0..8]，
     * 0：未知状态，1：信号灯未工作，2：红闪，3：红灯状态，
     * 4：绿灯待行状态，5：绿灯状态，6：受保护相位绿灯（箭头灯），7：黄灯状态，8：黄闪。
     * @param state
     * @return
     */
    private String transferLampGroup(int state) {
        String color = "";
        switch (state) {
            case 1://信号灯未工作
                color = "green";
                break;
            case 2:
                color = "redFlash";
                break;
            case 3:
                color = "red";
                break;
            case 4:
                color = "greenFlash";
                break;
            case 5:
                color = "green";
                break;
            case 6:
                color = "green";
                break;
            case 7:
                color = "yellow";
                break;
            case 8:
                color = "yellowFlash";
                break;
            default:
                color = "";
                break;
        }
        return color;
    }
    private static List<LightsStatusVO2> convertDirLampGroupMap2Entity(List<LightsStatusVO> lightsStatusVOS) throws Exception {
        List<LightsStatusVO2> listResult = null;
        try {
            listResult = new ArrayList<>(lightsStatusVOS.size());
            for (LightsStatusVO lightsStatusVO : lightsStatusVOS) {
                LightsStatusVO2 lightsStatusVO2 = new LightsStatusVO2();
                BeanUtils.copyProperties(lightsStatusVO, lightsStatusVO2);
                Map<String, Object> dirLampGroupMap = lightsStatusVO.getDirLampGroupMap();
                List<LightsStatusVO2.DirInfo> dirList = new ArrayList<>();
                Map<String, Object> phaseMap = lightsStatusVO.getPhaseMap();
                for (Map.Entry<String, Object> groupEntry : dirLampGroupMap.entrySet()) {
                    LightsStatusVO2.DirInfo dirInfo = new LightsStatusVO2.DirInfo();
                    String dir = groupEntry.getKey();
                    Map<Integer, String> turnColorMap = (Map<Integer, String>) groupEntry.getValue();
                    List<LightsStatusVO2.TurnInfo> turnList = new ArrayList<>();
                    for (Map.Entry<Integer, String> turnEntry : turnColorMap.entrySet()) {
                        setDirTurnInfo(phaseMap, dir, turnList, turnEntry);
                    }
                    dirInfo.setDir(dir);
                    dirInfo.setTurnList(turnList);
                    dirList.add(dirInfo);
                }
                lightsStatusVO2.setDirLampGroupMapList(dirList);
                listResult.add(lightsStatusVO2);
            }
        } catch (Exception e) {
            log.error("灯态推送转换V2异常：", e);
            throw new Exception(e);
        }
        return listResult;
    }

    /**
     * 设置方向转向信息实体
     *
     * @param phaseMap
     * @param dir
     * @param turnList
     * @param turnEntry
     */
    private static void setDirTurnInfo(Map<String, Object> phaseMap, String dir, List<LightsStatusVO2.TurnInfo> turnList, Map.Entry<Integer, String> turnEntry) {
        Integer turn = parseTurn(turnEntry);
        String color = turnEntry.getValue();
        LightsStatusVO2.TurnInfo turnInfo = new LightsStatusVO2.TurnInfo();
        turnInfo.setTurn(String.valueOf(turn));
        if (Objects.equals(turn, 20) && Objects.equals(color, "yellow")) { //行人黄灯时。绿闪
            turnInfo.setColor("greenFlash");
        } else {
            turnInfo.setColor(color);
        }
        setTurnInfoCounDown(phaseMap, dir, turn, turnInfo);
        turnList.add(turnInfo);
    }

    /**
     * 转换转向参数
     *
     * @param turnEntry
     * @return
     */
    private static Integer parseTurn(Map.Entry<Integer, String> turnEntry) {
        Integer turn = null;
        if (turnEntry.getKey() instanceof Integer) {
            turn = (Integer) turnEntry.getKey();
        } else {
            Object object = turnEntry.getKey();
            turn = Integer.parseInt(String.valueOf(object));
        }
        return turn;
    }

    /**
     * 设置方向转向倒计时时间
     *
     * @param phaseMap
     * @param dir
     * @param turn
     * @param turnInfo
     */
    private static void setTurnInfoCounDown(Map<String, Object> phaseMap, String dir, Integer turn, LightsStatusVO2.TurnInfo turnInfo) {
        String key = dir.concat(Constants.SEPARATOR_UNDER_LINE).concat(String.valueOf(turn));
        if (phaseMap.get(key) instanceof Map) {
            Map<Integer, Integer> turnMap = (Map<Integer, Integer>) phaseMap.get(key);
            if (Objects.nonNull(turnMap) && !turnMap.isEmpty()) {
                Integer countDown = turnMap.get(turn);
                turnInfo.setCountDown(countDown);
            }
        } else {
            Integer countDown = null;
            if (phaseMap.get(dir) instanceof Map) {
                Map<Integer, Integer> turnMap = (Map<Integer, Integer>) phaseMap.get(dir);
                countDown = turnMap.get(turn);
            } else {
                countDown = (Integer) phaseMap.get(key);
            }
            turnInfo.setCountDown(countDown);
        }
    }

    /**
     * 灯态数据发送kafka
     */
    private void sendKafka() {
        if (!produceListMap.isEmpty()) {
            List<LightsStatusVO2> list = new ArrayList<>(produceListMap.size());
            for (Map.Entry<String, List<LightsStatusVO2>> entry : produceListMap.entrySet()) {
                list.addAll(entry.getValue());
            }
            producerHandler.send(Constants.SIGNAL_LIGHT_HISTORY_KEY, list);
        }
    }

    /**
     * 灯态数据发送elastic
     */
    private void sendElastic() throws Exception {
        if (!produceListMap.isEmpty()) {
            List<LightsStatusVO2> list = new ArrayList<>(produceListMap.size());
            for (Map.Entry<String, List<LightsStatusVO2>> entry : produceListMap.entrySet()) {
                List<LightsStatusVO2> value = entry.getValue();
                for (LightsStatusVO2 lightsStatusVO2 : value) {
                    String json = JacksonUtils.getInstance().writeValueAsString(lightsStatusVO2);
                    GetIndexRequest getIndexRequest = new GetIndexRequest(Constants.SIGNAL_LIGHT_HISTORY_KEY);
                    boolean exists = client.indices().exists(getIndexRequest, RequestOptions.DEFAULT);
                    if (exists) {
                        IndexRequest indexRequest = new IndexRequest(Constants.SIGNAL_LIGHT_HISTORY_KEY);
                        indexRequest.source(json, XContentType.JSON);
                        IndexResponse response = client.index(indexRequest, RequestOptions.DEFAULT);
                    } else {
                        CreateIndexRequest createIndexRequest = new CreateIndexRequest(Constants.SIGNAL_LIGHT_HISTORY_KEY);
                        createIndexRequest.source(json, XContentType.JSON);
                        CreateIndexResponse createIndexResponse = client.indices().create(createIndexRequest, RequestOptions.DEFAULT);
                    }
                }
            }
        }
    }

    /**
     * 计算控制倒计时
     *
     * @param crossId
     * @param lightsStatusVO
     */
    private static void calculateControlCountDown(String crossId, LightsStatusVO lightsStatusVO) {
        Integer countTime = 0;
        // 特殊模式控制逻辑
        if (Arrays.asList(2, 3, 4, 9).contains(Integer.valueOf(lightsStatusVO.getRunMode()))) {
            if (runModeTimeMap.isEmpty() || Objects.isNull(runModeTimeMap.get(crossId))) {
                runModeTimeMap.put(crossId, countTime);
            } else {
                Integer countTimeCache = runModeTimeMap.get(crossId);
                runModeTimeMap.put(crossId, ++countTimeCache);
            }
            lightsStatusVO.setCycleCountDown(runModeTimeMap.get(crossId));
        } else if (StringUtils.equals(lightsStatusVO.getRunMode(), "11")) {
            runModeTimeMap.remove(crossId);
        }
        Constants.controlModeMap.put(crossId, lightsStatusVO.getRunMode());
    }

    /**
     * 不同系统获取统一路口灯态，统一处理
     *
     * @param evenWarnWebSocketMap
     * @return 返回类型是 Map<String, List<Map.Entry<String, RealTimeDataWebSocket>>>，表示按 crossId 分组后的连接条目。
     */
    private static Map<String, List<Map.Entry<String, RealTimeDataWebSocket>>> getCrossEntryMap(ConcurrentHashMap<String, RealTimeDataWebSocket> evenWarnWebSocketMap) {
        //map<crossId, List<Map.Entry<crossIdStr, RealTimeDataWebSocket>>>
        Map<String, List<Map.Entry<String, RealTimeDataWebSocket>>> tempMap = new ConcurrentHashMap<>();

        //遍历所有路口websocket
        for (Map.Entry<String, RealTimeDataWebSocket> item : evenWarnWebSocketMap.entrySet()) {
            String crossId = "";
            String crossIdStr = item.getKey();
            if (StringUtils.isNotBlank(crossIdStr)) {
                String[] split = crossIdStr.split(",");
                crossId = split[0];
            }
            List<Map.Entry<String, RealTimeDataWebSocket>> socketList = new ArrayList<>();
            if (tempMap.containsKey(crossId)) {
                socketList = tempMap.get(crossId);
                socketList.add(item);
            } else {
                socketList.add(item);
            }
            tempMap.put(crossId, socketList);
        }
        return tempMap;
    }

    /**
     * 插入路口灯态历史表
     *
     * @param crossId 路口ID
     * @param json    灯态JSON字符串
     * @throws Exception 异常处理
     */
    private void insertIntoHist(String crossId, String json) throws Exception {
        try {
            CrossLightsStatusHistPO crossLightsStatusHistPO = new CrossLightsStatusHistPO();
            crossLightsStatusHistPO.setCrossId(crossId);
            LocalDateTime now = LocalDateTime.now();
            Integer batchTime = (int) now.toEpochSecond(ZoneOffset.of("+8"));
            crossLightsStatusHistPO.setBatchTime(batchTime);
            crossLightsStatusHistPO.setLightsStatusJson(json);
            crossLightsStatusHistMapper.insertOne(crossLightsStatusHistPO);
        } catch (Exception e) {
            log.error("插入路口灯态历史表异常", e);
            throw new Exception(e);
        }
    }
}
